<?php

namespace Drupal\home_page_for_roles\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\path_alias\AliasManagerInterface;

/**
 * Controller for the homepage autocomplete route.
 */
class HomePageAutocompleteController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Constructs a HomePageAutocompleteController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The path alias manager.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    AliasManagerInterface $alias_manager
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->aliasManager = $alias_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('path_alias.manager')
    );
  }

  /**
   * Handles the autocomplete request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function handleAutocomplete(Request $request) {
    try {
      $matches = [];
      $string = $request->query->get('q');

      if (!empty($string)) {
        // Remove leading slash if present
        $search = ltrim($string, '/');
        
        // Get node storage
        $node_storage = $this->entityTypeManager->getStorage('node');
        
        // Build query
        $query = $node_storage->getQuery();
        $query->condition('status', 1)
              ->condition('title', '%' . $search . '%', 'LIKE')
              ->accessCheck(TRUE)
              ->sort('title', 'ASC')
              ->range(0, 10);

        $nids = $query->execute();

        if (!empty($nids)) {
          $nodes = $node_storage->loadMultiple($nids);
          foreach ($nodes as $node) {
            $internal_path = '/node/' . $node->id();
            $alias = $this->aliasManager->getAliasByPath($internal_path);
            
            // If there's no alias, use the internal path
            if (!$alias) {
              $alias = $internal_path;
            }
            
            $matches[] = [
              'value' => $alias,
              'label' => $node->label() . ' [' . $alias . ']'
            ];
          }
        }
      }

      return new JsonResponse($matches);
    }
    catch (\Exception $e) {
      \Drupal::logger('home_page_for_roles')->error($e->getMessage());
      return new JsonResponse(['error' => 'An error occurred while processing your request.'], 500);
    }
  }

}