<?php

namespace Drupal\home_page_for_roles\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Path\PathValidatorInterface;
use Drupal\Core\Path\AliasManagerInterface;
use Drupal\path_alias\AliasManagerInterface as PathAliasManagerInterface;

/**
 * Configure homepage settings for different roles.
 */
class HomePageForRolesSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The path validator.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected $pathValidator;

  /**
   * The path alias manager.
   *
   * @var \Drupal\Core\Path\AliasManagerInterface|\Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Constructs a HomePageForRolesSettingsForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Path\PathValidatorInterface $path_validator
   *   The path validator.
   * @param \Drupal\Core\Path\AliasManagerInterface|\Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The path alias manager.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    PathValidatorInterface $path_validator,
    $alias_manager
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->pathValidator = $path_validator;
    $this->aliasManager = $alias_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Handle both Drupal 9 and 10 alias manager services.
    $alias_manager = $container->has('path_alias.manager') 
      ? $container->get('path_alias.manager')
      : $container->get('path.alias_manager');

    return new static(
      $container->get('entity_type.manager'),
      $container->get('path.validator'),
      $alias_manager
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['home_page_for_roles.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'home_page_for_roles_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('home_page_for_roles.settings');

    // Default homepage for anonymous users
    $form['anonymous_homepage'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Anonymous Users Homepage'),
      '#description' => $this->t('Enter the path or start typing to search content'),
      '#default_value' => $config->get('anonymous_homepage') ?? '/',
      '#required' => TRUE,
      '#autocomplete_route_name' => 'home_page_for_roles.entity_autocomplete',
      '#autocomplete_minimum_characters' => 1,
    ];

    // Default homepage for authenticated users
    $form['authenticated_homepage'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default Authenticated Users Homepage'),
      '#description' => $this->t('Default homepage for logged-in users. Start typing to search content.'),
      '#default_value' => $config->get('authenticated_homepage') ?? '/user',
      '#required' => TRUE,
      '#autocomplete_route_name' => 'home_page_for_roles.entity_autocomplete',
      '#autocomplete_minimum_characters' => 1,
    ];

    // Get all roles except anonymous and authenticated
    $roles = \Drupal::entityTypeManager()->getStorage('user_role')->loadMultiple();
    unset($roles['anonymous']);
    unset($roles['authenticated']);
    
    $form['role_homepages'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Role-specific Homepages'),
      '#tree' => TRUE,
    ];

    foreach ($roles as $role_id => $role) {
      $form['role_homepages'][$role_id] = [
        '#type' => 'textfield',
        '#title' => $this->t('@role Homepage', ['@role' => $role->label()]),
        '#description' => $this->t('Homepage for users with the @role role. Start typing to search content.', ['@role' => $role->label()]),
        '#default_value' => $config->get("role_homepages.$role_id"),
        '#autocomplete_route_name' => 'home_page_for_roles.entity_autocomplete',
        '#autocomplete_minimum_characters' => 1,
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    
    // Validate and convert entity references to paths
    $this->validateAndConvertPath($form_state, 'anonymous_homepage');
    $this->validateAndConvertPath($form_state, 'authenticated_homepage');
    
    $role_homepages = $form_state->getValue('role_homepages');
    if ($role_homepages) {
      foreach ($role_homepages as $role_id => $path) {
        if (!empty($path)) {
          $this->validateAndConvertPath($form_state, ['role_homepages', $role_id]);
        }
      }
    }
  }

  /**
   * Validates and converts an entity reference to a path.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param string|array $field_path
   *   The field name or array path to the value.
   */
  protected function validateAndConvertPath(FormStateInterface $form_state, $field_path) {
    $value = is_array($field_path) 
      ? $form_state->getValue($field_path) 
      : $form_state->getValue($field_path);

    // Skip empty values except for required fields
    if (empty($value) && !in_array($field_path, ['anonymous_homepage', 'authenticated_homepage'])) {
      return;
    }

    // Handle special paths
    if (in_array($value, ['/', '/user'])) {
      return;
    }

    // Handle node title format (Title (123))
    if (preg_match('/^.+\(([0-9]+)\)$/', $value, $matches)) {
      try {
        $node = $this->entityTypeManager->getStorage('node')->load($matches[1]);
        if ($node) {
          $internal_path = '/node/' . $node->id();
          // Get the URL alias for this node
          $path = $this->aliasManager->getAliasByPath($internal_path);
          is_array($field_path)
            ? $form_state->setValue($field_path, $path)
            : $form_state->setValue($field_path, $path);
          return;
        }
      }
      catch (\Exception $e) {
        $this->setPathError($form_state, $field_path, $value);
        return;
      }
    }

    // Handle direct path input
    if (!empty($value)) {
      if (!$this->pathValidator->isValid($value)) {
        $this->setPathError($form_state, $field_path, $value);
      }
    }
  }

  /**
   * Sets a form error for an invalid path.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param string|array $field_path
   *   The field path.
   * @param string $value
   *   The invalid value.
   */
  protected function setPathError(FormStateInterface $form_state, $field_path, $value) {
    $field_name = is_array($field_path) ? end($field_path) : $field_path;
    $form_state->setErrorByName(
      $field_name,
      $this->t('The path %path does not exist on the site.', ['%path' => $value])
    );
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('home_page_for_roles.settings');
    
    $config->set('anonymous_homepage', $form_state->getValue('anonymous_homepage'))
           ->set('authenticated_homepage', $form_state->getValue('authenticated_homepage'))
           ->set('role_homepages', $form_state->getValue('role_homepages'))
           ->save();

    parent::submitForm($form, $form_state);
  }

}